Struct WebSocketConn
pub struct WebSocketConn { /* private fields */ }websockets only.Expand description
A struct that represents an specific websocket connection.
This can be thought of as a combination of a async_tungstenite::WebSocketStream and a
trillium::Conn, as it contains a combination of their fields and
associated functions.
The WebSocketConn implements Stream<Item=Result<Message, Error>>,
and can be polled with StreamExt::next
Implementations
impl WebSocketConn
impl WebSocketConn
pub async fn send_string(&mut self, string: String) -> Result<(), Error>
pub async fn send_string(&mut self, string: String) -> Result<(), Error>
send a Message::Text variant
pub async fn send_bytes(&mut self, bin: Vec<u8>) -> Result<(), Error>
pub async fn send_bytes(&mut self, bin: Vec<u8>) -> Result<(), Error>
send a Message::Binary variant
pub async fn feed(&mut self, message: Message) -> Result<(), Error>
pub async fn feed(&mut self, message: Message) -> Result<(), Error>
Enqueues a Message without immediately writing it to the socket
The message is encoded into an internal write buffer. The buffer is written out when it
fills, when this conn is next polled for an inbound message and none is immediately
available, or on flush or send. If you feed a message and
then await anything other than this conn, call flush first.
pub async fn flush(&mut self) -> Result<(), Error>
pub async fn flush(&mut self) -> Result<(), Error>
Writes any buffered outbound messages to the socket
pub fn set_peer_ip(&mut self, peer_ip: Option<IpAddr>) -> &mut WebSocketConn
pub fn set_peer_ip(&mut self, peer_ip: Option<IpAddr>) -> &mut WebSocketConn
Sets the peer ip for this conn
pub fn path(&self) -> &str
pub fn path(&self) -> &str
retrieves the path part of the request url, up to and excluding any query component
pub fn querystring(&self) -> &str
pub fn querystring(&self) -> &str
Retrieves the query component of the path, excluding ?. Returns
an empty string if there is no query component.
pub fn state<T>(&self) -> Option<&T>
pub fn state<T>(&self) -> Option<&T>
retrieve state from the state set that has been accumulated by
trillium handlers run on the trillium::Conn before it
became a websocket. see trillium::Conn::state for more
information
pub fn state_mut<T>(&mut self) -> Option<&mut T>
pub fn state_mut<T>(&mut self) -> Option<&mut T>
retrieve a mutable borrow of the state from the state set
pub fn insert_state<T>(&mut self, state: T) -> Option<T>
pub fn insert_state<T>(&mut self, state: T) -> Option<T>
inserts new state
returns the previously set state of the same type, if any existed
pub fn state_entry<T>(&mut self) -> Entry<'_, T>
pub fn state_entry<T>(&mut self) -> Entry<'_, T>
Returns an Entry for the state typeset that can be used with functions like
Entry::or_insert, Entry::or_insert_with, Entry::and_modify, and others.
pub fn take_state<T>(&mut self) -> Option<T>
pub fn take_state<T>(&mut self) -> Option<T>
take some type T out of the state set that has been
accumulated by trillium handlers run on the trillium::Conn
before it became a websocket. see trillium::Conn::take_state
for more information