WebSocketConn

Struct WebSocketConn 

pub struct WebSocketConn { /* private fields */ }
Available on crate feature websockets only.
A struct that represents an specific websocket connection.

This can be thought of as a combination of a async_tungstenite::WebSocketStream and a trillium::Conn, as it contains a combination of their fields and associated functions.

The WebSocketConn implements Stream<Item=Result<Message, Error>>, and can be polled with StreamExt::next

Implementations



impl WebSocketConn

pub async fn send_string(&mut self, string: String) -> Result<(), Error>

send a Message::Text variant

pub async fn send_bytes(&mut self, bin: Vec<u8>) -> Result<(), Error>

send a Message::Binary variant

pub async fn send(&mut self, message: Message) -> Result<(), Error>

Sends a Message to the client

pub fn swansong(&self) -> Swansong

retrieve a clone of the server’s Swansong

pub async fn close(&mut self) -> Result<(), Error>

close the websocket connection gracefully

pub fn headers(&self) -> &Headers

retrieve the request headers for this conn

pub fn peer_ip(&self) -> Option<IpAddr>

retrieves the peer ip for this conn, if available

pub fn set_peer_ip(&mut self, peer_ip: Option<IpAddr>) -> &mut WebSocketConn

Sets the peer ip for this conn

pub fn path(&self) -> &str

retrieves the path part of the request url, up to and excluding any query component

pub fn querystring(&self) -> &str

Retrieves the query component of the path, excluding ?. Returns an empty string if there is no query component.

pub fn method(&self) -> Method

retrieve the request method for this conn

pub fn state<T>(&self) -> Option<&T>
where T: Send + Sync + 'static,

retrieve state from the state set that has been accumulated by trillium handlers run on the trillium::Conn before it became a websocket. see trillium::Conn::state for more information

pub fn state_mut<T>(&mut self) -> Option<&mut T>
where T: Send + Sync + 'static,

retrieve a mutable borrow of the state from the state set

pub fn insert_state<T>(&mut self, state: T) -> Option<T>
where T: Send + Sync + 'static,

inserts new state

returns the previously set state of the same type, if any existed

pub fn state_entry<T>(&mut self) -> Entry<'_, T>
where T: Send + Sync + 'static,

Returns an Entry for the state typeset that can be used with functions like Entry::or_insert, Entry::or_insert_with, Entry::and_modify, and others.

pub fn take_state<T>(&mut self) -> Option<T>
where T: Send + Sync + 'static,

take some type T out of the state set that has been accumulated by trillium handlers run on the trillium::Conn before it became a websocket. see trillium::Conn::take_state for more information

pub fn take_inbound_stream( &mut self, ) -> Option<impl Stream<Item = Result<Message, Error>> + use<>>

take the inbound Message stream from this conn

pub fn inbound_stream( &mut self, ) -> Option<impl Stream<Item = Result<Message, Error>>>

borrow the inbound Message stream from this conn